@@ -65,7 +65,7 @@ print.summary.pagfl <- function(x, ...) {
   cat(x$convergence$convergence, paste0("(", x$convergence$iter, " iterations)\n"))
   cat("\nInformation criterion:\n")
   ic_vec <- c(IC = x$IC$IC, lambda = x$IC$lambda)
-  print(ic_vec)
+  print(round(ic_vec, 5))
   cat("\nResiduals:\n")
   resid_vec <- x$residuals
   quantile_vec <- round(stats::quantile(resid_vec, probs = c(0, .25, .5, .75, 1)), 5)
@@ -80,7 +80,7 @@ print.summary.pagfl <- function(x, ...) {
   cat("\nCoefficients:\n ")
   print(round(x$coefficients, 5))
   cat("\nResidual standard error:", round(x$r.se, 5), "on", x$r.df, "degrees of freedom\n")
-  cat("Mean squared error", round(x$IC$msr, 5))
+  cat("Mean squared error:", round(x$IC$msr, 5))
   cat("\nMultiple R-squared:", paste0(round(x$r.squared, 5), ","), "Adjusted R-squared:", round(x$adj.r.squared, 5), "\n")
 }
 
@@ -129,16 +129,21 @@ fitted.pagfl <- function(object, ...) {
     i_index = i_index,
     t_index = t_index
   )
-  colnames(fitted_df)[-1] <- object$args$labs$index
   plot_df <- fitted_df
-  plot_df$i_index <- as.character(plot_df$i_index)
-  plot_df$y <- object$model[[1]]
-  plot_df <- plot_df[order(plot_df$i_index), ]
-  y_name <- colnames(object$model)[1]
-  col_map <- c("red", "black")
-  names(col_map) <- c("fit", y_name)
+  colnames(fitted_df)[-1] <- object$args$labs$index
   # Plot the fit if feasible
   if (length(unique(i_index)) <= 20) {
+    if (!is.numeric(t_index)) {
+      suppressWarnings(t_index <- as.numeric(t_index))
+      if (all(is.na(t_index))) t_index <- as.integer(factor(object$args$labs$t))
+      plot_df$t_index <- t_index
+    }
+    plot_df$i_index <- as.character(plot_df$i_index)
+    plot_df$y <- object$model[[1]]
+    plot_df <- plot_df[order(plot_df$i_index), ]
+    y_name <- colnames(object$model)[1]
+    col_map <- c("red", "black")
+    names(col_map) <- c("fit", y_name)
     fit_plot <- gen_fit_plot_pagfl(plot_df = plot_df, y_name = y_name, col_map = col_map)
     print(fit_plot)
   }
